@@ -72,6 +72,7 @@ int Iflag;				/* command		*/
 int lflag;				/* [option]		*/
 int nflag;				/* [option]		*/
 int oflag;				/* [option]		*/
+int Oflag;				/* [option]		*/
 int pflag;				/* command		*/
 int Pflag;				/* command		*/
 int qflag;				/* [option]		*/
@@ -123,6 +124,7 @@ static struct option const long_options[] = {
 	{"ignore-case", no_argument, NULL, 'i'},
 	{"idutils", no_argument, NULL, 'I'},
 	{"other", no_argument, NULL, 'o'},
+	{"only-other", no_argument, NULL, 'O'},
 	{"print-dbpath", no_argument, NULL, 'p'},
 	{"path", no_argument, NULL, 'P'},
 	{"quiet", no_argument, NULL, 'q'},
@@ -165,7 +167,7 @@ main(int argc, char **argv)
 	char root[MAXPATHLEN+1];		/* root of source tree	*/
 	char dbpath[MAXPATHLEN+1];		/* dbpath directory	*/
 
-	while ((optchar = getopt_long(argc, argv, "ace:ifgGIlnopPqrstTuvx", long_options, &option_index)) != EOF) {
+	while ((optchar = getopt_long(argc, argv, "ace:ifgGIlnoOpPqrstTuvx", long_options, &option_index)) != EOF) {
 		switch (optchar) {
 		case 0:
 			break;
@@ -215,6 +217,9 @@ main(int argc, char **argv)
 		case 'o':
 			oflag++;
 			break;
+		case 'O':
+			Oflag++;
+			break;
 		case 'p':
 			pflag++;
 			setcom(optchar);
@@ -352,7 +357,7 @@ main(int argc, char **argv)
 	 */
 	getdbpath(cwd, root, dbpath, (pflag && vflag));
 	if (Iflag && !test("f", makepath(root, "ID", NULL)))
-		die("You must have id-utils's index at the root of source tree.");
+		die("You must have idutils's index at the root of source tree.");
 	/*
 	 * print dbpath or rootdir.
 	 */
@@ -422,7 +427,7 @@ main(int argc, char **argv)
 	 * The -t and -x option are given to priority more than
 	 * the --result option.
 	 */
-	if (fflag || gflag || Pflag || (nofilter & SORT_FILTER))
+	if (fflag || gflag || Pflag || Iflag || (nofilter & SORT_FILTER))
 		passthru = 1;
 	if (format == 0) {
 		if (tflag) { 			/* ctags format */
@@ -470,7 +475,7 @@ main(int argc, char **argv)
 		exit(0);
 	}
 	/*
-	 * exec lid(id-utils).
+	 * exec lid(idutils).
 	 */
 	if (Iflag) {
 		chdir(root);
@@ -598,7 +603,7 @@ printtag_using(const char *tag, const char *path, int lineno, const char *line)
 	filter_put(strbuf_value(sb));
 }
 /*
- * idutils:  lid(id-utils) pattern
+ * idutils:  lid(idutils) pattern
  *
  *	i)	pattern	POSIX regular expression
  *	i)	dbpath	GTAGS directory
@@ -615,7 +620,7 @@ idutils(const char *pattern, const char *dbpath)
 
 	lid = usable("lid");
 	if (!lid)
-		die("lid(id-utils) not found.");
+		die("lid(idutils) not found.");
 	/*
 	 * convert spaces into %FF format.
 	 */
@@ -635,7 +640,7 @@ idutils(const char *pattern, const char *dbpath)
 	strbuf_putc(ib, ' ');
 	strbuf_puts(ib, quote_string(pattern));
 	if (debug)
-		fprintf(stderr, "id-utils: %s\n", strbuf_value(ib));
+		fprintf(stderr, "idutils: %s\n", strbuf_value(ib));
 	if (!(ip = popen(strbuf_value(ib), "r")))
 		die("cannot execute '%s'.", strbuf_value(ib));
 	filter_open();
@@ -647,7 +652,7 @@ idutils(const char *pattern, const char *dbpath)
 		while (*p && *p != ':')
 			p++;
 		if ((xflag || tflag) && !*p)
-			die("invalid lid(id-utils) output format. '%s'", grep);
+			die("invalid lid(idutils) output format. '%s'", grep);
 		*p++ = 0;
 		if (lflag) {
 			if (!locatestring(path, localprefix + 2, MATCH_AT_FIRST))
@@ -666,11 +671,11 @@ idutils(const char *pattern, const char *dbpath)
 			while (*p && isdigit(*p))
 				p++;
 			if (*p != ':')
-				die("invalid lid(id-utils) output format. '%s'", grep);
+				die("invalid lid(idutils) output format. '%s'", grep);
 			*p++ = 0;
 			linenum = atoi(lno);
 			if (linenum <= 0)
-				die("invalid lid(id-utils) output format. '%s'", grep);
+				die("invalid lid(idutils) output format. '%s'", grep);
 			/*
 			 * print out.
 			 */
@@ -694,7 +699,7 @@ idutils(const char *pattern, const char *dbpath)
 			fprintf(stderr, "%d objects located", count);
 			break;
 		}
-		fprintf(stderr, " (using id-utils index in '%s').\n", dbpath);
+		fprintf(stderr, " (using idutils index in '%s').\n", dbpath);
 	}
 }
 /*
@@ -713,6 +718,7 @@ grep(const char *pattern, const char *dbpath)
 	const char *buffer;
 	int linenum, count;
 	int flags = 0;
+	int target = GPATH_SOURCE;
 	regex_t	preg;
 
 	/*
@@ -720,6 +726,10 @@ grep(const char *pattern, const char *dbpath)
 	 */
 	encode(edit, sizeof(edit), pattern);
 
+	if (oflag)
+		target = GPATH_BOTH;
+	if (Oflag)
+		target = GPATH_OTHER;
 	if (!Gflag)
 		flags |= REG_EXTENDED;
 	if (iflag)
@@ -729,7 +739,7 @@ grep(const char *pattern, const char *dbpath)
 	filter_open();
 	count = 0;
 
-	gp = gfind_open(dbpath, localprefix, oflag);
+	gp = gfind_open(dbpath, localprefix, target);
 	while ((path = gfind_read(gp)) != NULL) {
 		if (!(fp = fopen(path, "r")))
 			die("'%s' not found. Please remake tag files by invoking gtags(1).", path);
@@ -776,7 +786,12 @@ pathlist(const char *pattern, const char *dbpath)
 	const char *path, *p;
 	regex_t preg;
 	int count;
+	int target = GPATH_SOURCE;
 
+	if (oflag)
+		target = GPATH_BOTH;
+	if (Oflag)
+		target = GPATH_OTHER;
 	if (pattern) {
 		int flags = 0;
 		char edit[IDENTLEN+1];
@@ -803,7 +818,7 @@ pathlist(const char *pattern, const char *dbpath)
 	filter_open();
 	count = 0;
 
-	gp = gfind_open(dbpath, localprefix, oflag);
+	gp = gfind_open(dbpath, localprefix, target);
 	while ((path = gfind_read(gp)) != NULL) {
 		/*
 		 * skip localprefix because end-user doesn't see it.
@@ -1040,7 +1055,7 @@ tagsearch(const char *pattern, const char *cwd, const char *root, const char *db
 	/*
 	 * search in library path.
 	 */
-	if (getenv("GTAGSLIBPATH") && (count == 0 || Tflag) && !lflag && !rflag) {
+	if (db == GTAGS && getenv("GTAGSLIBPATH") && (count == 0 || Tflag) && !lflag) {
 		STRBUF *sb = strbuf_open(0);
 		char *libdir, *nextp = NULL;
 
