@@ -91,7 +91,7 @@ char global_path[MAXFILLEN];
 int ignore_case;
 
 void
-check_dbpath()
+check_dbpath(void)
 {
 	char cwd[MAXPATHLEN+1];
 	char root[MAXPATHLEN+1];
@@ -100,7 +100,7 @@ check_dbpath()
 	getdbpath(cwd, root, dbpath, vflag);
 }
 void
-get_global_path()
+get_global_path(void)
 {
 	const char *p;
 
@@ -154,7 +154,7 @@ main(int argc, char **argv)
 	return 0;
 }
 char *
-get_line()
+get_line(void)
 {
 	STATIC_STRBUF(sb);
 
@@ -166,7 +166,7 @@ get_line()
 	return strbuf_value(sb);
 }
 void
-update()
+update(void)
 {
 	STATIC_STRBUF(command);
 
@@ -179,7 +179,7 @@ update()
 	system(strbuf_value(command));
 }
 void
-print_case_distinction()
+print_case_distinction(void)
 {
 	if (vflag) {
 		const char *msg = ignore_case ? 
@@ -188,8 +188,30 @@ print_case_distinction()
 		fprintf(stderr, "gscope: %s\n", msg);
 	}
 }
+/*
+ * get pattern which match to #include lines.
+ *
+ *	i)	arg	include file name
+ *	r)		pattern which match to #include lines
+ */
+char *
+include_pattern(const char *arg)
+{
+#if defined(_WIN32)
+#define INCLUDE "^[ \t]*#[ \t]*include[ \t].*[\\\"\"</\\]%s[\\\"\">]"
+#elif defined(__DJGPP__)
+#define INCLUDE "^[ \t]*#[ \t]*include[ \t].*[\"</\\]%s[\">]"
+#else
+#define INCLUDE "^[ \t]*#[ \t]*include[ \t].*[\"</]%s[\">]"
+#endif
+	STATIC_STRBUF(pat);
+
+	strbuf_clear(pat);
+	strbuf_sprintf(pat, INCLUDE, quote_string(arg));
+	return strbuf_value(pat);
+}
 static void
-command_help()
+command_help(void)
 {
 	fprintf(stdout, "0<arg>: Find this C symbol\n");
 	fprintf(stdout, "1<arg>: Find this definition\n");
@@ -206,7 +228,7 @@ command_help()
 	fprintf(stdout, "h: Show help\n");
 }
 void
-command_loop()
+command_loop(void)
 {
 	int com = 0;
 	char *line;
@@ -221,6 +243,8 @@ command_loop()
 			ignore_case ^= 1;
 			print_case_distinction();
 			break;
+		case 'q':		/* q: quit the program */
+			return;
 		case 'r':		/* r: update tag file */
 			update();
 			break;
@@ -259,6 +283,11 @@ command_loop()
 static int
 execute_command(STRBUF *sb, const int com, const int opt, const char *arg)
 {
+#ifdef _WIN32
+#define QUOTE '"'
+#else
+#define QUOTE '\''
+#endif
 	STATIC_STRBUF(command);
 	STATIC_STRBUF(ib);
 	FILE *ip;
@@ -276,9 +305,9 @@ execute_command(STRBUF *sb, const int com, const int opt, const char *arg)
 			strbuf_putc(command, 'i');
 	}
 	strbuf_putc(command, ' ');
-	strbuf_putc(command, '\'');
+	strbuf_putc(command, QUOTE);
 	strbuf_puts(command, arg);
-	strbuf_putc(command, '\'');
+	strbuf_putc(command, QUOTE);
 	if (!(ip = popen(strbuf_value(command), "r")))
 		die("cannot execute '%s'.", strbuf_value(command));
 	if (vflag)
@@ -370,8 +399,7 @@ search(int com, const char *arg)
 		break;
 	case '8':		/* Find files #including this file */
 		opt = 'g';
-		snprintf(buf, sizeof(buf), "^[ \t]*#[ \t]*include[ \t].*[\"</]%s[\">]", quote_string(arg));
-		arg = buf;
+		arg = include_pattern(arg);
 		break;
 	}
 	/*
