@@ -1741,8 +1741,7 @@ yyreturn:
 
 
 void
-assembler(file)
-const char *file;
+assembler(const char *file)
 {
 	/* symbol search doesn't supported. */
 	if (sflag)
@@ -1763,8 +1762,7 @@ const char *file;
 }
 
 static void
-yyerror(s)
-const char *s;
+yyerror(const char *s)
 {
 
 }
