@@ -1,5 +1,5 @@
-/* C code produced by gperf version 3.0.1 */
-/* Command-line: gperf --language=C --struct-type --slot-name=name --hash-fn-name=php_hash --lookup-fn-name=php_lookup  */
+/* ANSI-C code produced by gperf version 3.0.1 */
+/* Command-line: gperf --language=ANSI-C --struct-type --slot-name=name --hash-fn-name=php_hash --lookup-fn-name=php_lookup  */
 /* Computed positions: -k'1-2,4-6,9' */
 
 #if !((' ' == 32) && ('!' == 33) && ('"' == 34) && ('#' == 35) \
@@ -26,7 +26,7 @@
       && ('w' == 119) && ('x' == 120) && ('y' == 121) && ('z' == 122) \
       && ('{' == 123) && ('|' == 124) && ('}' == 125) && ('~' == 126))
 /* The character set is not based on ISO-646.  */
-error "gperf generated tables don't work with this execution character set. Please report a bug to <bug-gnu-gperf@gnu.org>."
+#error "gperf generated tables don't work with this execution character set. Please report a bug to <bug-gnu-gperf@gnu.org>."
 #endif
 
 
@@ -134,9 +134,7 @@ inline
 #endif
 #endif
 static unsigned int
-php_hash (str, len)
-     register const char *str;
-     register unsigned int len;
+php_hash (register const char *str, register unsigned int len)
 {
   static unsigned short asso_values[] =
     {
@@ -200,9 +198,7 @@ php_hash (str, len)
 __inline
 #endif
 struct keyword *
-php_lookup (str, len)
-     register const char *str;
-     register unsigned int len;
+php_lookup (register const char *str, register unsigned int len)
 {
   static struct keyword wordlist[] =
     {
@@ -539,22 +535,16 @@ php_lookup (str, len)
   return 0;
 }
 
-int php_reserved_word(const char *, int);
 int
-php_reserved_word(str, len)
-	const char *str;
-	int len;
+php_reserved_word(const char *str, int len)
 {
 	struct keyword *keyword;
 
 	keyword = php_lookup(str, len);
 	return (keyword && IS_RESERVED_WORD(keyword->token)) ? keyword->token : 0;
 }
-int php_reserved_variable(const char *, int);
 int
-php_reserved_variable(str, len)
-	const char *str;
-	int len;
+php_reserved_variable(const char *str, int len)
 {
 	struct keyword *keyword;
 
