@@ -96,7 +96,7 @@ static int retry;
  * Each path name must start with "./".
  */
 static const char *
-getpath()
+getpath(void)
 {
 	static const char *buff;
 
@@ -113,7 +113,7 @@ getpath()
  * push back a path name.
  */
 static void
-ungetpath()
+ungetpath(void)
 {
 	retry = 1;
 }
@@ -701,7 +701,7 @@ makefileindex(const char *file, STRBUF *a_files)
 	indexlink = (Fflag) ? "../files" : "../mains";
 	src_count = 0;
 
-	gp = gfind_open(dbpath, NULL, other_files);
+	gp = gfind_open(dbpath, NULL, other_files ? GPATH_BOTH : GPATH_SOURCE);
 	/*
 	 * for collecting include files.
 	 */
