@@ -56,7 +56,7 @@ void src2html(const char *, const char *, int);
 int makedupindex(void);
 int makedefineindex(const char *, int, STRBUF *);
 int makefileindex(const char *, STRBUF *);
-void makeincludeindex();
+void makeincludeindex(void);
 
 #if defined(_WIN32) && !defined(__CYGWIN__)
 #define mkdir(path,mode) mkdir(path)
@@ -109,7 +109,6 @@ int symbol;				/* --symbol(-s) option          */
 int statistics;				/* --statistics option		*/
 
 int copy_files;				/* 1: copy tag files		*/
-int no_map_file;			/* 1: doesn't make map file	*/
 int no_order_list;			/* 1: doesn't use order list	*/
 int other_files;			/* 1: list other files		*/
 int enable_grep = 1;			/* 1: enable grep		*/
@@ -121,6 +120,7 @@ const char *id_value;
 const char *cgidir;
 const char *main_func = "main";
 const char *cvsweb_url;
+int use_cvs_module;
 const char *cvsweb_cvsroot;
 const char *gtagslabel;
 const char *title;
@@ -233,42 +233,66 @@ const char *include_file_suffixes = "h,hxx,hpp,H,inc.php";
 static const char *langmap = DEFAULTLANGMAP;
 
 static struct option const long_options[] = {
+	/*
+	 * These options have long name and short name.
+	 * We throw them to the processing of short options.
+	 */
         {"alphabet", no_argument, NULL, 'a'},
         {"compact", no_argument, NULL, 'c'},
-        {"dynamic", no_argument, NULL, 'D'},
         {"dbpath", required_argument, NULL, 'd'},
+        {"dynamic", no_argument, NULL, 'D'},
         {"form", no_argument, NULL, 'f'},
         {"frame", no_argument, NULL, 'F'},
+        {"func-header", optional_argument, NULL, 'h'},
         {"gtags", no_argument, NULL, 'g'},
-        {"func-header", no_argument, NULL, 'h'},
         {"icon", no_argument, NULL, 'I'},
+        {"line-number", optional_argument, NULL, 'n'},
         {"main-func", required_argument, NULL, 'm'},
-        {"line-number", no_argument, NULL, 'n'},
         {"other", no_argument, NULL, 'o'},
-        {"symbol", no_argument, NULL, 's'},
         {"secure-cgi", required_argument, NULL, 'S'},
+        {"symbol", no_argument, NULL, 's'},
         {"table-flist", optional_argument, NULL, 'T'},
         {"title", required_argument, NULL, 't'},
         {"verbose", no_argument, NULL, 'v'},
         {"warning", no_argument, NULL, 'w'},
-        {"xhtml", no_argument, NULL, 'x'},
+        {"xhtml", optional_argument, NULL, 'x'},
 
-        /* long name only */
-        {"action", required_argument, NULL, 1},
-        {"cvsweb", required_argument, NULL, 1},
-        {"cvsweb-cvsroot", required_argument, NULL, 1},
+        /*
+	 * The following are long name only.
+	 */
+	/* flag value */
         {"caution", no_argument, &caution, 1},
         {"debug", no_argument, &debug, 1},
-        {"gtagsconf", required_argument, NULL, 1},
-        {"gtagslabel", required_argument, NULL, 1},
-        {"id", required_argument, NULL, 1},
-        {"nocgi", no_argument, NULL, 1},
-        {"no-map-file", no_argument, &no_map_file, 1},
-        {"insert-header", required_argument, NULL, 1},
-        {"insert-footer", required_argument, NULL, 1},
+        {"disable-grep", no_argument, &enable_grep, 0},
+        {"full-path", no_argument, &full_path, 1},
+        {"nocgi", no_argument, &cgi, 0},
+        {"no-map-file", no_argument, &map_file, 0},
         {"statistics", no_argument, &statistics, 1},
+        {"table-list", no_argument, &table_list, 1},
         {"version", no_argument, &show_version, 1},
         {"help", no_argument, &show_help, 1},
+
+	/* accept value */
+#define OPT_ACTION		128
+#define OPT_CVSWEB		129
+#define OPT_CVSWEB_CVSROOT	130
+#define OPT_GTAGSCONF		131
+#define OPT_GTAGSLABEL		132
+#define OPT_NCOL		133
+#define OPT_ID			134
+#define OPT_INSERT_FOOTER	135
+#define OPT_INSERT_HEADER	136
+#define OPT_TABS		137
+        {"action", required_argument, NULL, OPT_ACTION},
+        {"cvsweb", required_argument, NULL, OPT_CVSWEB},
+        {"cvsweb-cvsroot", required_argument, NULL, OPT_CVSWEB_CVSROOT},
+        {"gtagsconf", required_argument, NULL, OPT_GTAGSCONF},
+        {"gtagslabel", required_argument, NULL, OPT_GTAGSLABEL},
+        {"ncol", required_argument, NULL, OPT_NCOL},
+        {"id", required_argument, NULL, OPT_ID},
+        {"insert-footer", required_argument, NULL, OPT_INSERT_FOOTER},
+        {"insert-header", required_argument, NULL, OPT_INSERT_HEADER},
+        {"tabs", required_argument, NULL, OPT_TABS},
         { 0 }
 };
 
@@ -409,7 +433,7 @@ generate_file(const char *dist, const char *file)
 		die("cannot create file '%s'.", file);
 	strbuf_reset(sb);
 	/*
-	 * Read templete file and evaluate macros.
+	 * Read template file and evaluate macros.
 	 */
 	while ((_ = strbuf_fgets(sb, ip, STRBUF_NOCRLF)) != NULL) {
 		const char *p;
@@ -639,7 +663,7 @@ makesearchpart(const char *action, const char *id, const char *target)
 		strbuf_puts_nl(sb, br);
 		strbuf_puts(sb, "('Other files' is effective only to 'Path name'");
 		if (enable_grep)
-			strbuf_puts(sb, "and 'Grep pattern'");
+			strbuf_puts(sb, " and 'Grep pattern'");
 		strbuf_puts_nl(sb, ".)");
 	}
 	strbuf_puts_nl(sb, gen_form_end());
@@ -788,7 +812,7 @@ makehtml(int total)
 	 * Create anchor stream for anchor_load().
 	 */
 	anchor_stream = tmpfile();
-	gp = gfind_open(dbpath, NULL, other_files);
+	gp = gfind_open(dbpath, NULL, other_files ? GPATH_BOTH : GPATH_SOURCE);
 	while ((path = gfind_read(gp)) != NULL) {
 		if (gp->type == GPATH_OTHER)
 			fputc(' ', anchor_stream);
@@ -803,7 +827,7 @@ makehtml(int total)
 	/*
 	 * For each path in GPATH, convert the path into HTML file.
 	 */
-	gp = gfind_open(dbpath, NULL, other_files);
+	gp = gfind_open(dbpath, NULL, other_files ? GPATH_BOTH : GPATH_SOURCE);
 	while ((path = gfind_read(gp)) != NULL) {
 		char html[MAXPATHLEN];
 
@@ -832,10 +856,13 @@ copyfile(const char *from, const char *to)
 	int ip, op, size;
 	char buf[8192];
 
-	ip = open(from, O_RDONLY);
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+	ip = open(from, O_RDONLY|O_BINARY);
 	if (ip < 0)
 		die("cannot open input file '%s'.", from);
-	op = open(to, O_WRONLY|O_CREAT|O_TRUNC, 0775);
+	op = open(to, O_WRONLY|O_CREAT|O_TRUNC|O_BINARY, 0775);
 	if (op < 0)
 		die("cannot create output file '%s'.", to);
 	while ((size = read(ip, buf, sizeof(buf))) != 0) {
@@ -1045,19 +1072,19 @@ configuration(int argc, char **argv)
 	strlimcpy(datadir, strbuf_value(sb), sizeof(datadir));
 	if (getconfn("ncol", &n)) {
 		if (n < 1 || n > 10)
-			warning("parameter 'ncol' ignored becase the value (=%d) is too large or too small.", n);
+			warning("parameter 'ncol' ignored because the value (=%d) is too large or too small.", n);
 		else
 			ncol = n;
 	}
 	if (getconfn("tabs", &n)) {
 		if (n < 1 || n > 32)
-			warning("parameter 'tabs' ignored becase the value (=%d) is too large or too small.", n);
+			warning("parameter 'tabs' ignored because the value (=%d) is too large or too small.", n);
 		else
 			tabs = n;
 	}
 	if (getconfn("flist_fields", &n)) {
 		if (n < 1)
-			warning("parameter 'flist_fields' ignored becase the value (=%d) is too large or too small.", n);
+			warning("parameter 'flist_fields' ignored because the value (=%d) is too large or too small.", n);
 		else
 			flist_fields = n;
 	}
@@ -1096,7 +1123,7 @@ configuration(int argc, char **argv)
 	if (getconfb("copy_files"))
 		copy_files = 1;
 	if (getconfb("no_map_file"))
-		no_map_file = 1;
+		map_file = 0;
 	strbuf_reset(sb);
 	if (getconfs("icon_spec", sb))
 		icon_spec = check_strdup(strbuf_value(sb));
@@ -1402,26 +1429,35 @@ main(int argc, char **argv)
 
 	while ((optchar = getopt_long(argc, argv, "acd:DfFghIm:noqsS:t:Tvwx", long_options, &option_index)) != EOF) {
 		switch (optchar) {
-                case 0:
-                case 1:
-			if (!strcmp("action", long_options[option_index].name))
-                                action_value = optarg;
-			else if (!strcmp("nocgi", long_options[option_index].name))
-                                cgi = 0;
-			else if (!strcmp("id", long_options[option_index].name))
-                                id_value = optarg;
-			else if (!strcmp("cvsweb", long_options[option_index].name))
-                                cvsweb_url = optarg;
-			else if (!strcmp("cvsweb-cvsroot", long_options[option_index].name))
-                                cvsweb_cvsroot = optarg;
-			else if (!strcmp("gtagsconf", long_options[option_index].name))
-				;	/*  --gtagsconf is estimated only once. */
-			else if (!strcmp("gtagslabel", long_options[option_index].name))
-				;	/* --gtagslabel is estimated only once. */
-			else if (!strcmp("insert-header", long_options[option_index].name))
-				insert_header = optarg;
-			else if (!strcmp("insert-footer", long_options[option_index].name))
-				insert_footer = optarg;
+		case 0:
+			/* already flags set */
+			break;
+		case OPT_ACTION:
+			action_value = optarg;
+			break;
+		case OPT_CVSWEB:
+			cvsweb_url = optarg;
+			break;
+		case OPT_CVSWEB_CVSROOT:
+			cvsweb_cvsroot = optarg;
+			break;
+		case OPT_GTAGSCONF:	/* --gtagsconf is estimated only once. */
+		case OPT_GTAGSLABEL:	/* --gtagslabel is estimated only once. */
+			break;
+		case OPT_ID:
+			id_value = optarg;
+			break;
+		case OPT_INSERT_FOOTER:
+			insert_footer = optarg;
+			break;
+		case OPT_INSERT_HEADER:
+			insert_header = optarg;
+			break;
+		case OPT_TABS:
+			if (atoi(optarg) > 0)
+				tabs = atoi(optarg);
+			else
+				die("--tabs option requires numeric value.");
                         break;
                 case 'a':
                         aflag++;
@@ -1445,8 +1481,17 @@ main(int argc, char **argv)
                         gflag++;
                         break;
                 case 'h':
-			if (definition_header == NO_HEADER)
-				definition_header = AFTER_HEADER;
+			definition_header = AFTER_HEADER;
+			if (optarg) {
+				if (!strcmp(optarg, "before"))
+					definition_header = BEFORE_HEADER;
+				else if (!strcmp(optarg, "right"))
+					definition_header = RIGHT_HEADER;
+				else if (!strcmp(optarg, "after"))
+					definition_header = AFTER_HEADER;
+				else
+					die("The option value of --func-header must be one of 'before', 'right' and 'after'.");
+			}
                         break;
                 case 'I':
                         Iflag++;
@@ -1456,6 +1501,12 @@ main(int argc, char **argv)
                         break;
                 case 'n':
                         nflag++;
+			if (optarg) {
+				if (atoi(optarg) > 0)
+					ncol = atoi(optarg);
+				else
+					die("The option value of --line-number must be numeric.");
+			}
                         break;
                 case 'o':
 			other_files = 1;
@@ -1469,8 +1520,12 @@ main(int argc, char **argv)
                         break;
                 case 'T':
 			table_flist = 1;
-			if (optarg && atoi(optarg) > 0)
-				flist_fields = atoi(optarg);
+			if (optarg) {
+				if (atoi(optarg) > 0)
+					flist_fields = atoi(optarg);
+				else
+					die("The option value of the --table-flist must be numeric.");
+			}
                         break;
                 case 't':
 			title = optarg;
@@ -1487,6 +1542,12 @@ main(int argc, char **argv)
                         break;
 		case 'x':
 			enable_xhtml = 1;
+			if (optarg) {
+				if (!strcmp("1.0", optarg) || !strcmp("1.1", optarg))
+					xhtml_version = optarg;
+				else
+					die("The option value of the --xhtml must be '1.0' or '1.1'.");
+			}
                         break;
                 default:
                         usage();
@@ -1497,8 +1558,6 @@ main(int argc, char **argv)
 		die("page header file '%s' not found.", insert_header);
 	if (insert_footer && !test("fr", insert_footer))
 		die("page footer file '%s' not found.", insert_footer);
-	if (no_map_file)
-		map_file = 0;
         argc -= optind;
         argv += optind;
         if (!av)
@@ -1506,6 +1565,7 @@ main(int argc, char **argv)
 
 	if (debug)
 		setdebug();
+	settabs(tabs);					/* setup tab skip */
         if (qflag) {
                 setquiet();
 		vflag = 0;
@@ -1567,6 +1627,8 @@ main(int argc, char **argv)
 	}
 	if (dynamic && Sflag)
 		die("Current implementation doesn't allow both -D(--dynamic) and the -S(--secure-cgi).");
+	if (cvsweb_url && test("d", "CVS"))
+		use_cvs_module = 1;
 	/*
 	 * decide directory in which we make hypertext.
 	 */
