@@ -206,10 +206,23 @@ echos(const char *s)
 {
         strbuf_puts(outbuf, s);
 }
-
 /*----------------------------------------------------------------------*/
 /* HTML output								*/
 /*----------------------------------------------------------------------*/
+/*
+ * Quote character with HTML's way.
+ */
+static const char *
+HTML_quoting(int c)
+{
+	if (c == '<')
+		return quote_little;
+	else if (c == '>')
+		return quote_great;
+	else if (c == '&')
+		return quote_amp;
+	return NULL;
+}
 /*
  * fill_anchor: fill anchor into file name
  *
@@ -539,13 +552,11 @@ missing_left(const char *word, int lineno)
 void
 put_char(int c)
 {
-        if (c == '<')
-		strbuf_puts(outbuf, quote_little);
-        else if (c == '>')
-		strbuf_puts(outbuf, quote_great);
-        else if (c == '&')
-		strbuf_puts(outbuf, quote_amp);
-        else
+	const char *quoted = HTML_quoting(c);
+
+	if (quoted)
+		strbuf_puts(outbuf, quoted);
+	else
 		strbuf_putc(outbuf, c);
 }
 /*
@@ -633,6 +644,70 @@ put_end_of_line(int lineno)
 	/* save for the other job in this module */
 	last_lineno = lineno;
 }
+/*
+ * Encode URL.
+ *
+ *	o)	sb	encoded URL
+ *	i)	url	URL
+ */
+static void
+encode(STRBUF *sb, const char *url)
+{
+	int c;
+
+	while ((c = (unsigned char)*url++) != '\0') {
+		if (isurlchar(c)) {
+			strbuf_putc(sb, c);
+		} else {
+			strbuf_putc(sb, '%');
+			strbuf_putc(sb, "0123456789abcdef"[c >> 4]);
+			strbuf_putc(sb, "0123456789abcdef"[c & 0x0f]);
+		}
+	}
+}
+/*
+ * get_cvs_module: return CVS module of source file.
+ *
+ *	i)	file		source path
+ *	o)	basename	If basename is not NULL, store pointer to
+ *				the last component of source path.
+ *	r)		!=NULL : relative path from repository top
+ *			==NULL : CVS/Repository is not readable.
+ */
+static const char *
+get_cvs_module(const char *file, const char **basename)
+{
+	const char *p;
+	STATIC_STRBUF(dir);
+	static char prev_dir[MAXPATHLEN+1];
+	STATIC_STRBUF(module);
+	FILE *ip;
+
+	strbuf_clear(dir);
+	p = locatestring(file, "/", MATCH_LAST);
+	if (p != NULL) {
+		strbuf_nputs(dir, file, p - file);
+		p++;
+	} else {
+		strbuf_putc(dir, '.');
+		p = file;
+	}
+	if (basename != NULL)
+		*basename = p;
+	if (strcmp(strbuf_value(dir), prev_dir) != 0) {
+		strlimcpy(prev_dir, strbuf_value(dir), sizeof(prev_dir));
+		strbuf_clear(module);
+		strbuf_puts(dir, "/CVS/Repository");
+		ip = fopen(strbuf_value(dir), "r");
+		if (ip != NULL) {
+			strbuf_fgets(module, ip, STRBUF_NOCRLF);
+			fclose(ip);
+		}
+	}
+	if (strbuf_getlen(module) > 0)
+		return strbuf_value(module);
+	return NULL;
+}
 /*
  *
  * src2html: convert source code into HTML
@@ -670,20 +745,22 @@ src2html(const char *src, const char *html, int notsource)
         fputs(fill_anchor(indexlink, src), out);
 	if (cvsweb_url) {
 		STATIC_STRBUF(sb);
-		const char *p;
+		const char *module, *basename;
 
 		strbuf_clear(sb);
 		strbuf_puts(sb, cvsweb_url);
-		for (p = src; *p; p++) {
-			int c = (unsigned char)*p;
-
-			if (c == '/' || isalnum(c))
-				strbuf_putc(sb, c);
-			else
-				strbuf_sprintf(sb, "%%%02x", c);
+		if (use_cvs_module
+		 && (module = get_cvs_module(src, &basename)) != NULL) {
+			encode(sb, module);
+			strbuf_putc(sb, '/');
+			encode(sb, basename);
+		} else {
+			encode(sb, src);
+		}
+		if (cvsweb_cvsroot) {
+			strbuf_puts(sb, "?cvsroot=");
+			strbuf_puts(sb, cvsweb_cvsroot);
 		}
-		if (cvsweb_cvsroot)
-			strbuf_sprintf(sb, "?cvsroot=%s", cvsweb_cvsroot);
 		fputs(quote_space, out);
 		fputs(gen_href_begin_simple(strbuf_value(sb)), out);
 		fputs(cvslink_begin, out);
@@ -712,29 +789,8 @@ src2html(const char *src, const char *html, int notsource)
 		fputs_nl(verbatim_begin, out);
 		last_lineno = 0;
 		while ((_ = strbuf_fgets(sb, in, STRBUF_NOCRLF)) != NULL) {
-			int dst = 0;
-
 			fputs(gen_name_number(++last_lineno), out);
-			for (; *_; _++) {
-				int c = *_;
-
-				if (c == '\t') {
-					do {
-						putc(' ', out);
-					} while (++dst % tabs);
-				} else {
-					if (c == '&')
-						fputs(quote_amp, out);
-					else if (c == '<')
-						fputs(quote_little, out);
-					else if (c == '>')
-						fputs(quote_great, out);
-					else
-						fputc(c, out);
-					dst++;
-				}
-			}
-			fputc('\n', out);
+			detab_replacing(out, _, HTML_quoting);
 		}
 		fputs_nl(verbatim_end, out);
 		strbuf_close(sb);
