@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003
+ * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2005
  *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
@@ -26,6 +26,7 @@
 #include <sys/stat.h>
 
 #include <assert.h>
+#include <ctype.h>
 #include <fcntl.h>
 #ifdef STDC_HEADERS
 #include <stdlib.h>
@@ -40,6 +41,7 @@
 #endif
 
 #include "char.h"
+#include "checkalloc.h"
 #include "dbop.h"
 #include "die.h"
 #include "locatestring.h"
@@ -48,6 +50,9 @@
 #include "test.h"
 
 int print_statistics = 0;
+
+#define ismeta(p)	(*((char *)(p)) == ' ')
+
 /*
  * dbop_open: open db database.
  *
@@ -60,11 +65,7 @@ int print_statistics = 0;
  *	r)		descripter for dbop_xxx()
  */
 DBOP *
-dbop_open(path, mode, perm, flags)
-	const char *path;
-	int mode;
-	int perm;
-	int flags;
+dbop_open(const char *path, int mode, int perm, int flags)
 {
 	DB *db;
 	int rw = 0;
@@ -93,7 +94,9 @@ dbop_open(path, mode, perm, flags)
 	info.psize = DBOP_PAGESIZE;
 	/*
 	 * accept user's request but needs 0.5MB at least.
+	 * The default value is 5MB.
 	 */
+	info.cachesize = 5000000;
 	if (getenv("GTAGSCACHE") != NULL)
 		info.cachesize = atoi(getenv("GTAGSCACHE"));
 	if (info.cachesize < 500000)
@@ -103,18 +106,21 @@ dbop_open(path, mode, perm, flags)
 	 * if unlink do job normally, those who already open tag file can use
 	 * it until closing.
 	 */
-	if (mode == 1 && test("f", path))
+	if (path != NULL && mode == 1 && test("f", path))
 		(void)unlink(path);
 	db = dbopen(path, rw, 0600, DB_BTREE, &info);
 	if (!db)
 		return NULL;
-	if (!(dbop = (DBOP *)malloc(sizeof(DBOP))))
-		die("short of memory.");
-	strlimcpy(dbop->dbname, path, sizeof(dbop->dbname));
+	dbop = (DBOP *)check_malloc(sizeof(DBOP));
+	if (path == NULL)
+		dbop->dbname[0] = '\0';
+	else
+		strlimcpy(dbop->dbname, path, sizeof(dbop->dbname));
 	dbop->db	= db;
 	dbop->openflags	= flags;
 	dbop->perm	= (mode == 1) ? perm : 0;
 	dbop->lastdat	= NULL;
+	dbop->lastsize	= 0;
 
 	return dbop;
 }
@@ -126,9 +132,7 @@ dbop_open(path, mode, perm, flags)
  *	r)		pointer to data
  */
 const char *
-dbop_get(dbop, name)
-	DBOP *dbop;
-	const char *name;
+dbop_get(DBOP *dbop, const char *name)
 {
 	DB *db = dbop->db;
 	DBT key, dat;
@@ -138,7 +142,8 @@ dbop_get(dbop, name)
 	key.size = strlen(name)+1;
 
 	status = (*db->get)(db, &key, &dat, 0);
-	dbop->lastdat	= (char *)dat.data;
+	dbop->lastdat = (char *)dat.data;
+	dbop->lastsize = dat.size;
 	switch (status) {
 	case RET_SUCCESS:
 		break;
@@ -147,7 +152,7 @@ dbop_get(dbop, name)
 	case RET_SPECIAL:
 		return (NULL);
 	}
-	return(dat.data);
+	return (dat.data);
 }
 /*
  * dbop_put: put data by a key.
@@ -157,10 +162,7 @@ dbop_get(dbop, name)
  *	i)	data	data
  */
 void
-dbop_put(dbop, name, data)
-	DBOP *dbop;
-	const char *name;
-	const char *data;
+dbop_put(DBOP *dbop, const char *name, const char *data)
 {
 	DB *db = dbop->db;
 	DBT key, dat;
@@ -185,6 +187,40 @@ dbop_put(dbop, name, data)
 		die("cannot write to database.");
 	}
 }
+/*
+ * dbop_put_withlen: put data by a key.
+ *
+ *	i)	dbop	descripter
+ *	i)	name	key
+ *	i)	data	data
+ *	i)	length	length of data
+ */
+void
+dbop_put_withlen(DBOP *dbop, const char *name, const char *data, int length)
+{
+	DB *db = dbop->db;
+	DBT key, dat;
+	int status;
+	int len;
+
+	if (!(len = strlen(name)))
+		die("primary key size == 0.");
+	if (len > MAXKEYLEN)
+		die("primary key too long.");
+	key.data = (char *)name;
+	key.size = strlen(name)+1;
+	dat.data = (char *)data;
+	dat.size = length;
+
+	status = (*db->put)(db, &key, &dat, 0);
+	switch (status) {
+	case RET_SUCCESS:
+		break;
+	case RET_ERROR:
+	case RET_SPECIAL:
+		die("cannot write to database.");
+	}
+}
 /*
  * dbop_delete: delete record by path name.
  *
@@ -192,9 +228,7 @@ dbop_put(dbop, name, data)
  *	i)	path	path name
  */
 void
-dbop_delete(dbop, path)
-	DBOP *dbop;
-	const char *path;
+dbop_delete(DBOP *dbop, const char *path)
 {
 	DB *db = dbop->db;
 	DBT key;
@@ -217,10 +251,7 @@ dbop_delete(dbop, path)
  *	i)	dat	data
  */
 void
-dbop_update(dbop, key, dat)
-	DBOP *dbop;
-	const char *key;
-	const char *dat;
+dbop_update(DBOP *dbop, const char *key, const char *dat)
 {
 	dbop_put(dbop, key, dat);
 }
@@ -239,11 +270,7 @@ dbop_update(dbop, key, dat)
  *	r)		data
  */
 const char *
-dbop_first(dbop, name, preg, flags)
-	DBOP *dbop;
-	const char *name;
-	regex_t *preg;
-	int	flags;
+dbop_first(DBOP *dbop, const char *name, regex_t *preg, int flags)
 {
 	DB *db = dbop->db;
 	DBT key, dat;
@@ -283,14 +310,16 @@ dbop_first(dbop, name, preg, flags)
 		for (status = (*db->seq)(db, &key, &dat, R_FIRST);
 			status == RET_SUCCESS;
 			status = (*db->seq)(db, &key, &dat, R_NEXT)) {
-			if (*((char *)key.data) == ' ')	/* meta record */
+			/* skip meta records */
+			if (ismeta(key.data))
 				continue;
 			if (preg && regexec(preg, (char *)key.data, 0, 0, 0) != 0)
 				continue;
 			break;
 		}
 	}
-	dbop->lastdat	= (char *)dat.data;
+	dbop->lastdat = (char *)dat.data;
+	dbop->lastsize = dat.size;
 	switch (status) {
 	case RET_SUCCESS:
 		break;
@@ -315,8 +344,7 @@ dbop_first(dbop, name, preg, flags)
  * Db_next always skip meta records.
  */
 const char *
-dbop_next(dbop)
-	DBOP *dbop;
+dbop_next(DBOP *dbop)
 {
 	DB *db = dbop->db;
 	int flags = dbop->ioflags;
@@ -325,9 +353,10 @@ dbop_next(dbop)
 
 	while ((status = (*db->seq)(db, &key, &dat, R_NEXT)) == RET_SUCCESS) {
 		assert(dat.data != NULL);
-		if (flags & DBOP_KEY && *((char *)key.data) == ' ')
+		/* skip meta records */
+		if (flags & DBOP_KEY && ismeta(key.data))
 			continue;
-		else if (*((char *)dat.data) == ' ')
+		else if (ismeta(dat.data))
 			continue;
 		if (flags & DBOP_KEY) {
 			if (!strcmp(dbop->prev, (char *)key.data))
@@ -337,6 +366,7 @@ dbop_next(dbop)
 			strlimcpy(dbop->prev, (char *)key.data, sizeof(dbop->prev));
 		}
 		dbop->lastdat	= (char *)dat.data;
+		dbop->lastsize	= dat.size;
 		if (flags & DBOP_PREFIX) {
 			if (strncmp((char *)key.data, dbop->key, dbop->keylen))
 				return NULL;
@@ -359,26 +389,112 @@ dbop_next(dbop)
  *	r)		last data
  */
 const char *
-dbop_lastdat(dbop)
-	DBOP *dbop;
+dbop_lastdat(DBOP *dbop, int *size)
 {
+	if (size)
+		*size = dbop->lastsize;
 	return dbop->lastdat;
 }
+/*
+ * get_flag: get flag value
+ */
+const char *
+dbop_getflag(DBOP *dbop)
+{
+	int size;
+	const char *dat = dbop_lastdat(dbop, &size);
+	const char *flag = "";
+	/*
+	 * Dat format is like follows.
+	 * dat 'xxxxxxx\0ffff\0'
+	 *      (data)   (flag)
+	 */
+	if (dat) {
+		int i = strlen(dat) + 1;
+		if (size > i)
+			flag = dat + i;
+	}
+	return flag;
+}
+/*
+ * dbop_getoption: get option
+ */
+const char *
+dbop_getoption(DBOP *dbop, const char *key)
+{
+	static char buf[1024];
+	const char *p;
+
+	if ((p = dbop_get(dbop, key)) == NULL)
+		return NULL;
+	if (dbop->lastsize <= strlen(key))
+		die("illegal format (dbop_getoption).");
+	for (p += strlen(key); *p && isspace((unsigned char)*p); p++)
+		;
+	strlimcpy(buf, p, sizeof(buf));
+	return buf;
+}
+/*
+ * dbop_putoption: put option
+ */
+void
+dbop_putoption(DBOP *dbop, const char *key, const char *string)
+{
+	char buf[1024];
+
+	if (string)
+		snprintf(buf, sizeof(buf), "%s %s", key, string);
+	else
+		snprintf(buf, sizeof(buf), "%s", key);
+	dbop_put(dbop, key, buf);
+}
+/*
+ * dbop_getversion: get format version
+ */
+int
+dbop_getversion(DBOP *dbop)
+{
+	int format_version = 1;			/* default format version */
+	const char *p;
+
+	if ((p = dbop_getoption(dbop, VERSIONKEY)) != NULL)
+		format_version = atoi(p);
+	return format_version;
+}
+/*
+ * dbop_putversion: put format version
+ */
+void
+dbop_putversion(DBOP *dbop, int version)
+{
+	char number[32];
+
+	snprintf(number, sizeof(number), "%d", version);
+	dbop_putoption(dbop, VERSIONKEY, number);
+}
 /*
  * dbop_close: close db
  * 
  *	i)	dbop	dbop descripter
  */
 void
-dbop_close(dbop)
-	DBOP *dbop;
+dbop_close(DBOP *dbop)
 {
 	DB *db = dbop->db;
 
+#ifdef USE_DB185_COMPAT
 	(void)db->close(db);
-	if (dbop->openflags & DBOP_REMOVE)
-		(void)unlink(dbop->dbname);
-	else if (dbop->perm && chmod(dbop->dbname, dbop->perm) < 0)
-		die("cannot change file mode.");
+#else
+	/*
+	 * If DBOP_REMOVE is specified, omit writing to the disk in __bt_close().
+	 */
+	(void)db->close(db, (dbop->openflags & DBOP_REMOVE || dbop->dbname[0] == '\0') ? 1 : 0);
+#endif
+	if (dbop->dbname[0] != '\0') {
+		if (dbop->openflags & DBOP_REMOVE)
+			(void)unlink(dbop->dbname);
+		else if (dbop->perm && chmod(dbop->dbname, dbop->perm) < 0)
+			die("cannot change file mode.");
+	}
 	(void)free(dbop);
 }
