@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 1997, 1998, 1999 Shigio Yamaguchi
- * Copyright (c) 1999, 2000, 2001, 2002, 2003 Tama Communications Corporation
+ * Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003
+ *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,47 +32,36 @@
 static int quiet;
 static int verbose;
 static int debug;
-static void (*exit_proc)();
+static void (*exit_proc)(void);
 
 void
-setquiet()
+setquiet(void)
 {
 	quiet = 1;
 }
 void
-setverbose()
+setverbose(void)
 {
 	verbose = 1;
 }
 void
-setdebug()
+setdebug(void)
 {
 	debug = 1;
 }
 void
-sethandler(proc)
-	void (*proc)();
+sethandler(void (*proc)(void))
 {
 	exit_proc = proc;
 }
 void
-#ifdef HAVE_STDARG_H
 die(const char *s, ...)
-#else
-die(s, va_alist)
-	char *s;
-	va_dcl;
-#endif
 {
 	va_list ap;
 
 	if (!quiet) {
 		fprintf(stderr, "%s: ", progname);
-#ifdef HAVE_STDARG_H
 		va_start(ap, s);
-#else
-		va_start(ap);
-#endif
 		(void)vfprintf(stderr, s, ap);
 		va_end(ap);
 		fputs("\n", stderr);
@@ -85,24 +74,13 @@ die(s, va_alist)
 }
 
 void
-#ifdef HAVE_STDARG_H
 die_with_code(int n, const char *s, ...)
-#else
-die_with_code(n, s, va_alist)
-	int n;
-	char *s;
-	va_dcl;
-#endif
 {
 	va_list ap;
 
 	if (!quiet) {
 		fprintf(stderr, "%s: ", progname);
-#ifdef HAVE_STDARG_H
 		va_start(ap, s);
-#else
-		va_start(ap);
-#endif
 		(void)vfprintf(stderr, s, ap);
 		va_end(ap);
 		fputs("\n", stderr);
@@ -114,45 +92,25 @@ die_with_code(n, s, va_alist)
 	exit(n);
 }
 void
-#ifdef HAVE_STDARG_H
 message(const char *s, ...)
-#else
-message(s, va_alist)
-	char *s;
-	va_dcl;
-#endif
 {
 	va_list ap;
 
 	if (!quiet && verbose) {
-#ifdef HAVE_STDARG_H
 		va_start(ap, s);
-#else
-		va_start(ap);
-#endif
 		(void)vfprintf(stderr, s, ap);
 		va_end(ap);
 		fputs("\n", stderr);
 	}
 }
 void
-#ifdef HAVE_STDARG_H
 warning(const char *s, ...)
-#else
-warning(s, va_alist)
-	char *s;
-	va_dcl;
-#endif
 {
 	va_list ap;
 
 	if (!quiet) {
 		fputs("Warning: ", stderr);
-#ifdef HAVE_STDARG_H
 		va_start(ap, s);
-#else
-		va_start(ap);
-#endif
 		(void)vfprintf(stderr, s, ap);
 		va_end(ap);
 		fputs("\n", stderr);
