@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 1997, 1998, 1999, 2000
+ * Copyright (c) 1996, 1997, 1998, 1999, 2000, 2006
  *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
@@ -15,15 +15,20 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifndef _TAB_H_
 #define _TAB_H_
 
+#include <stdio.h>
+
 void settabs(int);
 void detab(FILE *, const char *);
 void entab(char *);
+size_t read_file_detabing(char *, size_t, FILE *, int *, int *);
+void detab_replacing(FILE *op, const char *buf, const char *(*replace)(int c));
+
 
 #endif /* ! _TAB_H_ */
