@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999, 2000
+ * Copyright (c) 1997, 1998, 1999, 2000, 2006
  *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
@@ -15,8 +15,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -49,21 +49,20 @@
  *	r)	0: is not binary, 1: is binary
  */
 static int
-is_binary(path)
-	const char *path;
+is_binary(const char *path)
 {
 	int ip;
 	char buf[32];
 	int i, c, size;
 
-	ip = open(path, 0);
+	ip = open(path, O_RDONLY);
 	if (ip < 0)
 		die("cannot open file '%s' in read mode.", path);
 	size = read(ip, buf, sizeof(buf));
 	close(ip);
 	if (size < 0)
 		return 1;
-	if (!strncmp(buf, "!<arch>", 7))
+	if (locatestring(buf, "!<arch>", MATCH_AT_FIRST))
 		return 1;
 	for (i = 0; i < size; i++) {
 		c = (unsigned char)buf[i];
@@ -92,9 +91,7 @@ is_binary(path)
  * You can specify more than one character. It assumed 'AND' test.
  */
 int
-test(flags, path)
-	const char *flags;
-	const char *path;
+test(const char *flags, const char *path)
 {
 	static struct stat sb;
 	int c;
@@ -150,3 +147,19 @@ test(flags, path)
 	}
 	return 1;
 }
+/*
+ * filesize: get file size in bytes.
+ *
+ *	i)	path	path of file
+ *	r)		!= -1: file size
+ *			== -1: file not found
+ */
+int
+filesize(const char *path)
+{
+	struct stat sb;
+
+	if (stat(path, &sb) < 0)
+		return -1;
+	return sb.st_size;
+}
