@@ -15,8 +15,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifndef _TOKEN_H_
@@ -27,14 +27,14 @@
 
 #define SYMBOL		0
 
-extern const unsigned char *sp, *cp, *lp;
+extern const char *sp, *cp, *lp;
 extern int lineno;
 extern int crflag;
 extern int cmode;
 extern int cppmode;
 extern int ymode;
-extern unsigned char token[MAXTOKEN];
-extern unsigned char curfile[MAXPATHLEN];
+extern char token[MAXTOKEN];
+extern char curfile[MAXPATHLEN];
 extern int continued_line;
 
 #define nextchar() \
@@ -43,8 +43,10 @@ extern int continued_line;
 			EOF : \
 			(lineno++, *cp == 0 ? \
 				(lp = cp, cp = NULL, continued_line = 0, '\n') : \
-				*cp++)) : \
-		(*cp == 0 ? (lp = cp, cp = NULL, continued_line = 0, '\n') : *cp++))
+				(unsigned char)*cp++)) : \
+		(*cp == 0 ? \
+			(lp = cp, cp = NULL, continued_line = 0, '\n') : \
+			(unsigned char)*cp++))
 #define atfirst (sp && sp == (cp ? cp - 1 : lp))
 
 int opentoken(const char *);
