@@ -25,6 +25,7 @@
 #include <stdlib.h>
 #endif
 #include <stdio.h>
+#include "checkalloc.h"
 #include "die.h"
 #include "varray.h"
 
@@ -84,14 +85,10 @@ static int debug = 0;
  *	r)	vb	VARRAY structure
  */
 VARRAY *
-varray_open(size, expand)
-	int size;
-	int expand;
+varray_open(int size, int expand)
 {
-	VARRAY *vb = (VARRAY *)malloc(sizeof(VARRAY));
+	VARRAY *vb = (VARRAY *)check_malloc(sizeof(VARRAY));
 
-	if (vb == NULL)
-		die("short of memory.");
 	if (size < 1)
 		die("varray_open: size < 1.");
 	if (expand < 0)
@@ -115,10 +112,7 @@ varray_open(size, expand)
  * This procedure doesn't operate the contents of the array.
  */
 void *
-varray_assign(vb, index, force)
-	VARRAY *vb;
-	int index;
-	int force;
+varray_assign(VARRAY *vb, int index, int force)
 {
 	if (index < 0)
 		die("varray_assign: illegal index value.");
@@ -144,9 +138,9 @@ varray_assign(vb, index, force)
 		 * Therefore, we cannot use realloc(NULL, ...).
 		 */
 		if (vb->vbuf == NULL)
-			vb->vbuf = (char *)malloc(vb->size * vb->alloced);
+			vb->vbuf = (char *)check_malloc(vb->size * vb->alloced);
 		else
-			vb->vbuf = (char *)realloc(vb->vbuf, vb->size * vb->alloced);
+			vb->vbuf = (char *)check_realloc(vb->vbuf, vb->size * vb->alloced);
 		if (debug)
 			fprintf(stderr, "Expanded: from %d to %d.\n", old_alloced, vb->alloced);
 	}
@@ -162,8 +156,7 @@ varray_assign(vb, index, force)
  * This procedure doesn't operate the contents of the array.
  */
 void *
-varray_append(vb)
-	VARRAY *vb;
+varray_append(VARRAY *vb)
 {
 	return varray_assign(vb, vb->length, 1);
 }
@@ -173,8 +166,7 @@ varray_append(vb)
  *	i)	vb	VARRAY structure
  */
 void
-varray_reset(vb)
-	VARRAY	*vb;
+varray_reset(VARRAY *vb)
 {
 	vb->length = 0;
 }
@@ -184,8 +176,7 @@ varray_reset(vb)
  *	i)	vb	VARRAY structure
  */
 void
-varray_close(vb)
-	VARRAY	*vb;
+varray_close(VARRAY *vb)
 {
 	if (vb) {
 		if (vb->vbuf)
