@@ -15,8 +15,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -30,8 +30,8 @@
 #include "version.h"
 
 const char *copy = "\
-Copyright (c) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005\n\
-	Tama Communications Corporation\n\
+Copyright (c) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,\n\
+	2006 Tama Communications Corporation\n\
 GNU GLOBAL comes with NO WARRANTY, to the extent permitted by law.\n\
 You may redistribute copies of GNU GLOBAL under the terms of the\n\
 GNU General Public License. For more information about these matters,\n\
@@ -49,9 +49,7 @@ get_version(void)
  * version: print version information.
  */
 void
-version(name, verbose)
-	const char *name;
-	const int verbose;
+version(const char *name, const int verbose)
 {
 	if (name == NULL)
 		name = progname;
